# coding: utf8
from maze_keeper import MazeObjects
import os, time


class Visualization:
    """Handles visualization of the maze and traces through the maze. Visualization is to the console window."""
    def __init__(self, layout, speed=0.2):
        self.layout = layout
        self.M = len(layout[0])
        self.N = len(layout[1])
        self.draw_mapping = {MazeObjects.EMPTY: ' ',
                             MazeObjects.OBSTACLE: '█',
                             MazeObjects.GOLD: 'G',
                             MazeObjects.START: 'x'}
        self.speed = speed

    def line(self, top=True):
        corners = ('╔', '╗') if top else ('╚', '╝')
        return corners[0] + '═' * self.N + corners[1]

    @staticmethod
    def cls():
        os.system('cls' if os.name == 'nt' else 'clear')

    def show(self, highlight_cord=(), highlight='A'):
        output = [self.line(top=True), '\n']
        for r, row in enumerate(self.layout):
            output.append('║')
            for c, cell in enumerate(row):
                if (r, c) in highlight_cord:
                    output.append(highlight[0])
                else:
                    output.append(self.draw_mapping[cell])
            output.append('║\n')
        output.append(self.line(top=False))
        output.append('\n')

        print(''.join(output))

    def show_trace(self, trace):
        for step, position in enumerate(trace):
            self.cls()
            print('\t Step: {}'.format(step))
            self.show(highlight_cord=[position], highlight='A')
            time.sleep(self.speed)


if __name__ == '__main__':
    agent_trace = [(0, 2), (0, 3), (0, 4), (1, 4), (2, 4), (2, 3), (2, 2), (2, 1), (2, 0), (3, 0), (4, 0), (4, 1), (4, 2)]

    E, O, G, S = MazeObjects.EMPTY, MazeObjects.OBSTACLE, MazeObjects.GOLD, MazeObjects.START
    SAMPLE_LAYOUT = [[E, O, E, E, E],
                     [E, O, E, O, E],
                     [S, O, E, O, G],
                     [E, O, E, O, E],
                     [E, E, E, E, E]]

    v = Visualization(SAMPLE_LAYOUT)
    v.show()
    v.show_trace(agent_trace)
